/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.enumChoice
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.extensions.component.ShopAction
import net.moonleay.bedge.extensions.component.ShopItem
import net.moonleay.bedge.util.EmbedColor
import net.moonleay.bedge.util.MessageUtil

class ShopExtension : Extension() {

    override val name = "shop"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        publicSlashCommand(::ShopArguments) {
            name = "shop"
            description = "Buy stuff with your items"

            this.action {
                val user = this.user.asUser()
                val ud = UserRepository.getUserByID(user.id.value)

                if (ud == null) {
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(EmbedColor.ERROR,
                                "You don't have an account here!",
                                "And therefore I cannot show you our shop.\n" +
                                        "You can register by running `/time`.",
                                user.username)
                        )
                    }
                    return@action
                }
                when(this.arguments.action){
                    ShopAction.LIST_OFFERS -> {
                        var description =   "Your Coins: ${ud.coins}\n" +
                                "Your lvl: ${ud.level}\n\n" +
                                "item :: price :: required level\n"

                        ShopItem.entries.forEach { itm ->
                            description += "\"${itm.readableName}\" :: ${itm.cost} coins :: ${itm.requiredLevel}\n"
                        }

                        this.respond {
                            this.embeds.add(
                                MessageUtil.getEmbed(
                                    EmbedColor.INFO,
                                    "The Shop",
                                    description,
                                    user.username
                                )
                            )
                        }
                    }
                    ShopAction.BUY -> {
                        var description = ""

                    }
                }
            }
        }
    }

    inner class ShopArguments : Arguments() {

        val action by enumChoice<ShopAction> {
            this.name = "action"
            this.description = "What you want to do"
        }
    }
}
